
class: center, middle

# PF Basics
  
  
### Section 1

---
## It all started with a copyright violation

- OpenBSD up to 2.9 (2001) used Darren Reed's IPFilter

  - Almost, but not quite BSD licensed

  - No right to distribute changed versions



- IPFilter removed on May 29th, 2001

  - First commit of the new PF code June 24 2001 at 19:48:58 UTC

  - OpenBSD 3.0 release pushed to Dec 1 by the extra effort


- *License audit* of src tree + ports tree followed

But let's jump to the present -

???



---
## Network design 101

How to build a maintainable network:

- Start with something simple

- Think dual stack (IPv6 and IPv4)

- Users
  - Access to local and remote services
    - and their security requirements

- The services you offer and the ones you consume
  - i.e: SMTP, WWW, DNS

- **Keep it simple, not stupid** 

---
## But first, how do we build rulesets?

The basic _rule_ format is, 

 **verb** _criteria_ **actions** ... options

```shell
 pass in on egress proto tcp to egress port ssh
```

```shell
 match out on egress nat-to egress
```

```shell
 block all
```


You can do complete rulesets with only these, but you may also want to declare things ->

( **egress** ? -> the _interface group_ consisting of interfaces that have a default route )

---
## Next, declarations

Declaring objects your rules will use:

**Macros:**

```shell
macro-name = "definition"
```

**Tables:**

```shell
table <tablename> 
```

**Queues:**


```shell
queue queuename on interface bandwidth number ...
```

You can also *set* options.

We'll be returning to all of those, in the meantime [man pf.conf](http://man.openbsd.org/pf.conf) is always your friend.

---
## Block by default

- A firewall is supposed to block by default, right?

- Let's start with the ruleset for a client

- Useful baseline:
  - block by default
  - pass traffic we generate

```shell
    block
    pass from (self)
```

---
## In practice

- On my laptop in my home network, that expands to:

```shell
    $ doas pfctl -vnf /etc/pf.conf
    block drop all
    pass inet6 from ::1 to any flags S/SA
    pass on lo0 inet6 from fe80::1 to any flags S/SA
    pass on iwm0 inet6 from fe80::a2a8:cdff:fe63:abb9 to any flags S/SA
    pass inet6 from 2001:470:28:658:a2a8:cdff:fe63:abb9 to any flags S/SA
    pass inet6 from 2001:470:28:658:8c43:4c81:e110:9d83 to any flags S/SA
    pass inet from 127.0.0.1 to any flags S/SA
    pass inet from 192.168.103.126 to any flags S/SA
```
???

It would be nice in this slide to ask questions, to see the general knowledge about protocols:

- IPv6 LLA
- IPv6 GUA
- IPv6 GUA with privacy extensions
- IPv4 Private address

This can give us an idea of how to proceed with IPv6 depending on the level of the participants.


---
##   Anatomy of a ruleset 

- Ruleset evaluated top to bottom

- **Last match wins**  (unless _quick_ is used)

- Stateful by default

- For more details, see [man pf.conf](http://man.openbsd.org/pf.conf)
  - http://man.openbsd.org/pf.conf

---
## A ruleset can contain

- **Options**
	- General configuration

- **Macros**
	- Content to be expanded
  
- **Tables**
	- When macros fall short (for IP addresses only)

- **Redirections**

- **Rules**
	- How PF should behave

- The order of the items in the ruleset used to be important, but not anymore

---

## Options

- General configuration for pf

- Useful for debugging, applying default timeout values, etc.

```shell
    set limit states 100000
```

- or

```shell
    set debug debug
    set loginterface dc0
    set timeout tcp.first 120 
    set timeout tcp.established 86400 
    set timeout { adaptive.start 6000, adaptive.end 12000 }
```

---

## Macros

- Content that will later be expanded in context

- Name must start with a letter, digit, or underscore

- Useful to keep rulesets simple and clean

```shell
    ext_if = "kue0" 
    all_ifs = "{" $ext_if lo0 "}" 
    pass out on $ext_if from any to any 
    pass in  on $ext_if proto tcp from any to any port 25
```

- Could also be port or address ranges

```shell
   bacula_ports = "9101:9103"
   dmz_hosts = "192.0.2.1 - 192.0.2.254"
```

???

If we have more slide space, then separate the second example in its own slide

---
##   Tables

- When IP address list macros grow too long, use tables instead

- Hold only addresses and networks
  - No port ranges

- Provide fast lookups
  
```shell
   table &lt;bruteforce&gt; persist counters
```

- They can be used in rules

```shell
   block from &lt;bruteforce&gt;
```

- Unlike macros with port or address lists, _do not_ expand to multiple rules
  - More optimised ruleset

---

## Tables continued


- Content can be loaded from files or from CLI

```shell
    table <popflooders> persist counters file "/var/tmp/popflooders"
```

- use [pfctl](http://man.openbsd.org/pfctl) to add/remove entries from command line

```shell
   $ doas pfctl -t bruteforce -T add 192.0.2.11 2001:db8::dead:beef:baad:f00d
```
  
- optionally maintained by daemons such as [spamd](http://man.openbsd.org/spamd), [bgpd](http://man.openbsd.org/bgpd) and [dhcpd](http://man.openbsd.org/dhcpd)


---

class: center, middle

##   Implementing your spec

### How would you do this in your home/soho infrastructure ?

???

We ask the participants how they think they would implement this in their infrastructure.

This is to create discussion, and hopefully make it so that we all learn a bit by trial-and-error.

Let's put out the question, wait for some time to see if anyone comes up with a nice description, and then walk them through what is a sensible setup.

This is in preparation for the first exercise which comes next.

---
