
class: center, middle

# Hosting Services
  
  
### Section 3

---

## Hosting services behind your gateway


- Now you actually want some 'pass in on egress' rules :)

- Get your specifications clear, put in writing:

  - your services and the ports they use
    - the names could be in **/etc/services** already

  - the hosts (IP addresses) that run the services

  - decide who/what/where to be reachable for/from

- Check services requiring extra help (i.e. proxying).

---
## Proxies and other helpers

- OpenBSD comes with several proxies and other service helper programs in the base system:

  - [ftp-proxy](http://man.openbsd.org/ftp-proxy) (you guessed it)

  - [relayd](http://man.openbsd.org/relayd) (load balancing and lots more)

  - [spamd](http://man.openbsd.org/spamd) (if you run SMTP - annoy spammers)

<br><br><br>

- There are also such things as squid and varnish (web proxies) in packages

<br><br><br>

- Clients and services in the same subnet? Or do the DMZs?
  
---
## DMZ, defined

### '*D*e-*M*ilitarized *Z*one'

- when a group of host needs special treatment 

<br><br>

- attached to separate interfaces(s), separate sub-rulesets

<br><br>

- And **YES**, you can have several

<br><br>

- Think multiple customers, *N* environments each

---
background-image: url(images/exercise3.png)
class: right
## DMZ, illustrated




---
##Allowing some services in

```shell 
    ext_if=bge0 # adjust to what your system has
    int_if=bge1 # adjust to what your system has

    client_out = "{ ftp-data, ftp, ssh, domain, pop3, auth, nntp, http, \
                https, 2628, 5999, 8000, 8080 }"
    udp_services = "{ domain, ntp }"

*   webserver = "192.0.2.227"
*   webports = "{ http, https }"
*   emailserver = "192.0.2.225"
*   email = "{ smtp, pop3, imap, imap3, imaps, pop3s }"
*   nameservers = "{ 192.0.2.221, 192.0.2.223 }"

    match out on egress inet nat-to ($ext_if)
    block
    pass quick proto { tcp, udp } to port $udp_services keep state
    pass proto tcp from $int_if:network to port $client_out
    pass proto tcp to self port ssh

*   pass proto tcp to $webserver port $webports 
*   pass proto tcp to $emailserver port $email 
*   pass log proto tcp from $emailserver to port smtp 
*   pass inet proto { tcp, udp } to $nameservers port domain

```

Now try loading this with *pfctl -vnf /etc/pf.conf* and see what this expands to (you can also fetch [pf.services01.conf](samples/pf.services01.conf) and try)

---
## Tackling noise (attacks) with state-tracking options

### Scenario: ssh bruteforcers

In our previous ruleset, add 

```shell 
table &lt;bruteforce&gt; persist counters
block from &lt;bruteforce&gt;
```

and change the **ssh** rule to

```shell 
* pass proto tcp to port ssh flags S/SA keep state \
*   (max-src-conn 15, max-src-conn-rate 2/10, overload <bruteforce> flush global)

```

- Tune to taste
- More info on options in [man pf.conf](http://man.openbsd.org/pf.conf)
- Remember *[pfctl](http://man.openbsd.org/pfctl) expire*

???
max-src-conn - see man page but max number of simultaneous connections from one host
max-src-conn-rate - number of new connections per number of seconds 

---

## Tackling noise (attacks) with state-tracking options

### Scenario: Wordpress site

- Wordpress is a usual target for many different attacks

- -> Make the attackers suffer by forcing them through smaller "windows"
  - Use tables and block by connection rates

- Mix and match settings, consult [man pf.conf](http://man.openbsd.org/pf.conf) and remember *[pfctl](http://man.openbsd.org/pfctl) expire*

- Also see [Forcing the password gropers through a smaller hole with OpenBSD's PF queues](http://bsdly.blogspot.ca/2017/04/forcing-password-gropers-through.html) + [The Book of PF](https://www.nostarch.com/pf3)

---

## Scenario: Wordpress site

- You can re-use the *bruteforce* table or make a separate one, like

```shell 
  table &lt;web_brutes&gt; persist counters
  block from &lt;web_brutes&gt;
```

- Now change the **$webports** rule to

```shell 
* pass proto tcp to port $webports flags S/SA keep state \
*   (max-src-conn 15, max-src-conn-rate 5/10, overload <web_brutes> flush global)

```

- Alternatively, on a machine (the gateway) that does not run Wordpress, do

```shell 
  grep wp-login /var/www/logs/access.log | awk '{print $1}' | \ 
    sort -u | xargs doas pfctl -t web_brutes -T add
```

- (**Wordpressers**: Do these look right? Consult web logs and [tcpdump](http://man.openbsd.org/tcpdump) output.)


???
max-src-conn - see man page but max number of simultaneous connections from one host
max-src-conn-rate - number of new connections per number of seconds 
You could also look into max-src-nodes, max-src-states

---

## Annoying spammers with spamd

- [spamd(8)](http://man.openbsd.org/spamd) is good, clean, fun
<br><br>
- Speaks enough SMTP to do [greylisting](https://en.wikipedia.org/wiki/Greylisting)
<br><br>
- Can tarpit known bad senders and generate blacklists by greytrapping
<br><br>
- Default [spamd.conf](http://man.openbsd.org/spamd.conf) gives you one blacklist import and basics
  - (**Hint:** no real SMTP service required)
<br><br>
- You can even generate your own blacklists by *greytrapping* via *non-deliverable* spamtrap addresses in your own domain(s)
<br><br>
- Also see: 
  - [The Book of PF](https://www.nostarch.com/pf3)
  - [In The Name Of Sane Email: Setting Up OpenBSD's spamd(8) With Secondary MXes](http://bsdly.blogspot.ca/2012/05/in-name-of-sane-email-setting-up-spamd.html)
  - [Maintaining A Publicly Available Blacklist](http://bsdly.blogspot.ca/2013/04/maintaining-publicly-available.html)

---
## Annoying spammers with spamd

- Set up tables
- Divert traffic to the spamd process
- Only let the "good guys" pass to the real SMTP daemon

```shell 
table <spamd-white> persist
table <nospamd> persist file "/etc/mail/nospamd"
pass in on egress proto tcp to any port smtp divert-to 127.0.0.1 port spamd
pass in on egress proto tcp from <nospamd> to any port smtp
pass in log on egress proto tcp from <spamd-white> to any port smtp
pass out log on egress proto tcp to any port smtp
```

- and then run

```shell
$ doas rcctl enable spamd
$ doas rcctl start spamd
```

**TIP:** check out [smtpctl spf walk <_nospamd_domains.txt_](https://man.openbsd.org/smtpctl) to feed your _nospamd_ table from live [SPF](https://en.wikipedia.org/wiki/Sender_Policy_Framework) data (in OpenBSD 6.3 onwards)

---
## Scenario: SYN flood vs syncookies

A common Denial-of-Service (DOS) technique is to send large numbers of SYNs from spoofed addresses, filling up the state table.

In OpenBSD 6.3 and newer we have the [pf.conf](http://man.openbsd.org/pf.conf) option

```shell 
set syncookies
```

The default is off, if you enable with 

```shell 
set syncookies on 
```

**all** SYNs get SYNACK answer, but no resources allocated until ACK received (pending match to **pass** rule)

The other option is **adaptive** with syncookies used only when half open TCP connections reach the **start** percentage, until the **end** level is reached

```shell 
set syncookies adaptive (start 29%, end 15%) 
```


---
## Consider: what about that separate DMZ?

- What do you need?
<br><br><br>
- IP addresses: Segment off or allocate separate address ranges
<br><br><br>
- Attach each segment to separate interface, VLAN
<br><br><br>
- Do the ruleset surgery
  - which services do you run, where
  - do you need renumbering?
  - what traffic (in and out) is actually required?

???



---

class: center, middle


# Questions ?
  
???

Let's ask if there are any questions before continuing.  Make sure we have everyone onboard.

---

