
class: center, middle

# Gateway, NAT and Redirects 
  
  
### Section 2

---
## Easy! How do we improve on this?

- Make a 'firewall':

  - a point of policy enforcement

  - a gateway

  - filter for other hosts

  - redirection tricks

---
## Introducing NAT

- **N**etwork **A**ddress **T**ranslation ([RFC1631](https://tools.ietf.org/html/rfc1631) onwards)

- -> 'Hide' several hosts behind 1 public address, using [RFC1918](https://tools.ietf.org/html/rfc1918) addresses

- Modern PF has _nat-to_ on 'pass' and 'match' rules:

```shell
	match out on $extif inet nat-to ($extif)
```

- *Neat trick*: egress is the interface group that has a default route, you can filter on it


```shell
	match out on egress inet nat-to (egress)
```



- In modern networks we **should** (also) have IPv6 (inet6)

???

NAT, the stopgap measure that's old enough to drink, more than 22 years old.  NAT was created as a temporary measure that hasn't been replaced by now.  Not even IPv6.

We haven't discusses "egress" yet, so this is the right time to introduce it.

Spend some time discussing it, along with the rest of the NAT specifications.

Unfortunately, there's also NAT for IPv6, called NAT66

---
## A (filtering) Gateway

_"I decide which packets pass"_

#### Enable forwarding:

- Temporarily set from command line with [sysctl](http://man.openbsd.org/sysctl):

```shell 
    # sysctl net.inet.ip.forwarding=1 
    # sysctl net.inet6.ip6.forwarding=1 
```

- Make permanent in [/etc/sysctl.conf](http://man.openbsd.org/sysctl.conf)

```shell 
    net.inet.ip.forwarding=1
    net.inet6.ip6.forwarding=1
```


---
## The minimal gateway

- Do you *NAT* for IPv4? Of course you do.

- Do you run IPv6? Of course you do.

```shell 
    ext_if=bge0
    int_if=bge1
    match out on egress inet nat-to ($ext_if)
    block all
    pass proto tcp from { self, $int_if:network }
```

- The "pass" rule, withouth _inet_ or _inet6_ applies to both

**Keep in mind**: This is a point of policy enforcement

---
## A Point of policy enforcement

- Now some policy, and macros

```shell 
  ext_if=bge0
  int_if=bge1

  client_out = "{ ftp-data, ftp, ssh, domain, pop3, auth, nntp, http, \
      https, 2628, 5999, 8000, 8080 }
  udp_services = "{ domain, ntp }"

  match out on egress inet nat-to ($ext_if)
* block
* pass quick proto { tcp, udp } to port $udp_services keep state
*
* pass proto tcp from $int_if:network to port $client_out
*
* pass proto tcp to self port ssh
```

- What services do your clients consume?

???

Log to on the system we have for showing out, and then show the rules there and how they expand to different parts.

---
## Letting dhcpd(8) direct access

OpenBSD [dhcpd(8)](http://man.openbsd.org/dhcpd) can interact with your ruleset via tables:

*/etc/rc.conf.local*
```shell 
dhcpd_flags="-L leased_ip_table -A abandoned_ip_table -C changed_ip_table bge1"
```

```shell 
  ext_if=bge0
  int_if=bge1
* table &lt;abandoned_ip_table&gt; persist counters
* table &lt;changed_ip_table&gt; persist counters
* table &lt;leased_ip_table&gt; persist counters

  client_out = "{ ftp-data, ftp, ssh, domain, pop3, auth, nntp, http, \
                  https, 2628, 5999, 8000, 8080 }"
  udp_services = "{ domain, ntp }"

  match out on egress inet nat-to ($ext_if)
  block
  pass quick proto { tcp, udp } to port $udp_services keep state
* pass proto tcp from &lt;leased_ip_table&gt; to port $client_out
  pass proto tcp to self port ssh
```
**=>** only pass traffic from hosts with active leases from *me*

???

Maybe move this slide to a later section.

---
## Redirects (and divert-to)

  Modern PF has two classes of redirect

  * **rdr-to** on match and pass rules - rewrite destination address while filtering (locally or even to other hosts)

```shell 
      pass in on egress to port www rdr-to $webserver
```


  * **divert-to** on match and pass rules - [divert()](http://man.openbsd.org/divert) socket for local use

```shell 
      pass in on egress to port smtp divert-to 127.0.0.1 port spamd
```


---

## FTP Proxy

- If your users need to access FTP services, [ftp-proxy](http://man.openbsd.org/ftp-proxy) is what you need

- FTP does not easily pass through a block firewall, some help is needed


```shell 
$ doas rcctl enable ftpproxy6
```

- or for IPv4

```shell 
$ doas rcctl enable ftpproxy
```

- and then add an anchor and divert rules to your config

```shell 
anchor "ftp-proxy/*"
  ... 
pass in quick inet proto  tcp to port ftp divert-to 127.0.0.1 port 8021
pass in quick inet6 proto tcp to port ftp divert-to ::1 port 8021
pass out proto tcp from $proxy to port ftp
```

There is even a reverse mode (**-R**) for when you host FTP servers, see [man ftp-proxy](http://man.openbsd.org/ftp-proxy)

---
