
class: center, middle

# Traffic Shaping
  
  
### Section 4

---

## Traffic shaping

- OpenBSD has three separate shaping techniques:

  - *priorities* (set prio), introduced in OpenBSD 5.0

  - *queues*, introduced in OpenBSD 5.5, and

  - *flows*, introduced in OpenBSD 6.2 (aka FQ-CoDel)
<br><br>
- **Remember:**

  - Traffic shaping is about *dropping packets*

  - But is only relevant when there's a *reason* to start dropping

  - Then you get to pick which ones using the traffic shaping tools
<br><br>
- Works only one way, **outbound**

---
## Traffic shaping with priorities  

- In OpenBSD, every packet has a priority
<br><br>
- Possible values are 0 (garbage) through 7 (want!)
  - Default for most traffic is 3.
<br><br>
- So if you want *all* ssh traffic to move ahead of other traffic you could do a
<br><br>
```shell
  pass proto tcp to port ssh set prio 6
```
<br><br>
- and assign specific, non-default ( != 3 ) values to others.

---
## Beating the FIFO with prio  

- By default, packets are serviced on a first come, first served basis
  - or 'First in, First out' (FIFO).
<br>
- But: TCP wants ACKs for sent packets within a reasonable time
  - Otherwise the packet is considered lost, and retransmit will follow (transfer stalls).
<br>
- ACKs are tiny and have their TOS set to lowdelay, and this trick will cheat the FIFO:
<br>
```shell
   match out on $ext_if proto tcp from $ext_if set prio (3, 7)
   match in  on $ext_if proto tcp to $ext_if set prio (3, 7)
```
<br>
as per the [man page](http://man.openbsd.org/pf.conf),
<br><br>
*If two priorities are given, TCP ACKs with no data payload and packets which have a TOS of lowdelay will be assigned to the second one. Packets with a higher priority number are processed first, and packets with the same priority are processed in the order in which they are received.*
<br>

- See [Prioritizing empty TCP ACKs with pf and ALTQ](http://www.benzedrine.ch/ackpri.html) by Daniel Hartmeier for the ALTQ way

---
## FQ-CoDel flows for fair bandwidth sharing

Introduced in [OpenBSD 6.2](https://www.openbsd.org/62.html), the FQ-CoDel algorithm (see [RFC8290](https://tools.ietf.org/html/rfc8290)) defines **flows** to set up fair sharing for a specified number of simultaneous connections

Enable for your configuration with something like

```shell
queue outq on bge0 bandwidth 18M max 18M flows 1024 qlimit 1024 \
               default
```

Estimate your approximate high number of simultaneously actively transmitting sessions, put that number in (up to 32767) 

---
##  Shaping with HFSC queues - fixed sizes

- When priorities don't quite cut it, you can slice your bandwidth into queues.
<br><br>
- For static shaping, give bandwidth values in absolute values:
<br><br>
- Only leaf queues can be assigned traffic
 - make sure allocations sum up to parent queue allocation
<br><br>
- Unless quotas approach saturation, no actual shaping (dropping) will take place
  
---
## Shaping with HFSC queues - fixed sizes
<br><br><br>

```shell
  queue main on $ext_if bandwidth 20M
      queue defq parent main bandwidth 3600K default
      queue ftp parent main bandwidth 2000K
      queue udp parent main bandwidth 6000K
      queue web parent main bandwidth 4000K
      queue ssh parent main bandwidth 4000K
        queue ssh_interactive parent ssh bandwidth 800K
        queue ssh_bulk parent ssh bandwidth 3200K
  queue icmp parent main bandwidth 400K
```

---
##  Shaping with HFSC queues - fixed assignment

- You can either do queue assignment with match rules:
  
```shell
match log quick on $ext_if proto tcp to port ssh \
	queue (ssh_bulk, ssh_interactive)
match in quick on $ext_if proto tcp to port ftp queue ftp
match in quick on $ext_if proto tcp to port www queue http
match out on $ext_if proto udp queue udp
match out on $ext_if proto icmp queue icmp
```
- treat filtering (block, pass) in separate rules,

-  *or*

  - append 'set queue' to individual pass rules

- Any traffic not explicitly assigned goes in the default queue
  
---
##  Shaping with HFSC queues - flexible allocations (min, max, burst)

You can build in flexibility:
  
```shell
queue rootq on $ext_if bandwidth 20M
        queue main parent rootq bandwidth 20479K min 1M max 20479K qlimit 100
             queue qdef parent main bandwidth 9600K min 6000K max 18M default
             queue qweb parent main bandwidth 9600K min 6000K max 18M
             queue qpri parent main bandwidth 700K min 100K max 1200K
             queue qdns parent main bandwidth 200K min 12K burst 600K for 3000ms
        queue spamd parent rootq bandwidth 1K min 0K max 1K qlimit 300
```
Note here the added *min* and *max* values: combinded queue bandwidth can exceed actual sum; this gets you upper and lower bound within physical limits.

  *burst N for M* - allow bursts of that size and length

  *qlimit* - size of the queue's holding buffer - larger values *may* delay (packets are kept longer before sending)

---
## Systat

- Available on all the BSDs
  - OpenBSD version has added functionalities

- Offers view into your system's traffic
  - Live queues, states, rules
  
- Check how your rules are behaving on your system in realtime

- More info in the man page [systat(1)](http://man.openbsd.org/systat)

---
## queue monitoring - systat queues

<br><br><br>

```shell
   1 users Load 2.56 2.27 2.28                        skapet.bsdly.net 20:55:50

QUEUE                BW SCH  PRI    PKTS   BYTES   DROP_P   DROP_B QLEN BOR SUS  P/S   B/S
rootq on bge0       20M                0       0        0        0    0            0     0
 main               20M                0       0        0        0    0            0     0
  qdef               9M          6416363   2338M      136    15371    0          462 30733
  qweb               9M           431590 144565K        0        0    0          0.6   480
  qpri               2M          2854556 181684K        5      390    0           79  5243
  qdns             100K           802874  68379K        0        0    0          0.6    52
 spamd               1K           596022  36021K  1177533 72871514  299            2   136
```  
or

---
## queue monitoring - pfctl   

```shell
$ doas pfctl -vsq
  [ pkts:          0  bytes:          0  dropped pkts:      0 bytes:      0 ]
  [ qlength:   0/ 50 ]
queue rootq on bge0 bandwidth 20M qlimit 50
  [ pkts:          0  bytes:          0  dropped pkts:      0 bytes:      0 ]
  [ qlength:   0/ 50 ]
queue main parent rootq bandwidth 20M, min 1M, max 20M qlimit 100
  [ pkts:          0  bytes:          0  dropped pkts:      0 bytes:      0 ]
  [ qlength:   0/100 ]
queue qdef parent main bandwidth 9M, min 8M, max 18M default qlimit 50
  [ pkts:    6517150  bytes: 2458545319  dropped pkts:    136 bytes:  15371 ]
  [ qlength:   0/ 50 ]
queue qweb parent main bandwidth 9M, min 8M, max 18M qlimit 50
  [ pkts:     431741  bytes:  148072219  dropped pkts:      0 bytes:      0 ]
  [ qlength:   0/ 50 ]
queue qpri parent main bandwidth 2M, min 700K, max 2M burst 4M for 3000ms qlimit 50
  [ pkts:    2855418  bytes:  186101241  dropped pkts:      5 bytes:    390 ]
  [ qlength:   0/ 50 ]
queue qdns parent main bandwidth 100K, min 12K burst 600K for 3000ms qlimit 50
  [ pkts:     803548  bytes:   70079760  dropped pkts:      0 bytes:      0 ]
  [ qlength:   0/ 50 ]
queue spamd parent rootq bandwidth 1K, max 1K qlimit 300
  [ pkts:     596424  bytes:   36910940  dropped pkts: 1178456 bytes: 72928604 ]
  [ qlength: 300/300 ]
```  

add another v ([pfctl](http://man.openbsd.org/pfctl) -vvsq) for continuously updating display
 
---

class: center, middle


# Questions ?
  
???

Let's ask if there are any questions before continuing.  Make sure we have everyone onboard.

---



