class: center, middle

# Troubleshooting
  
  
### Section 6

#### "It's all your fault. Until you track down and fix the root cause."
---
## Troubleshooting 101: ICMP(v6)

- ICMP: **I**nternet **C**ontrol **M**essage **P**rotocol

- The *ping of death* scare is almost over, let's enable [ping](http://man.openbsd.org/ping):
<br><br>
```shell 
  icmp_types = "{ echoreq, unreach }"
  
  pass inet proto icmp all icmp-type $icmp_types keep state
  
  pass inet proto icmp from $localnet icmp-type $icmp_types
  pass inet proto icmp to $ext_if icmp-type $icmp_types
  pass inet6 proto icmp6 from $localnet icmp6-type $icmp6_types
  pass inet6 proto icmp6 to $ext_if icmp6-type $icmp6_types
```
<br><br>
- **echoreq**: lets [ping](http://man.openbsd.org/ping) do its thing

- **unreach**: lets you do _path MTU discovery_ (PMTUD)

---

## Troubleshooting 101: Statistics

- Statistics can be had with **[pfctl](http://man.openbsd.org/pfctl) -s info**

For statistics (bytes/packets passed per rule) attach _labels_ per rule

```shell 
pass log proto { tcp, udp } to $emailserver port smtp label "mail-in"
pass log proto { tcp, udp } from $emailserver to port smtp label "mail-out"
```

```shell
$ doas pfctl -vs rules
pass inet proto tcp from any to 192.0.2.225 port = smtp flags S/SA keep state label "mail-in"
[ Evaluations: 1664158 Packets: 1601986 Bytes: 763762591 States: 0 ]
[ Inserted: uid 0 pid 24490 ]
pass inet proto tcp from 192.0.2.225 to any port = smtp flags S/SA keep state label "mail-out"
[ Evaluations: 2814933 Packets: 2711211 Bytes: 492510664 States: 0 ]
[ Inserted: uid 0 pid 24490 ] 
```

---
## Troubleshooting 101: Statistics

- If you need to pass the data to a script
 - Or a database
 - A graphing engine

<br><br><br>

```shell
$ doas pfctl -zvsl
mail-in 1664158 1601986 763762591 887895 682427415 714091 81335176
mail-out 2814933 2711211 492510664 1407278 239776267 1303933 252734397
```

---
## Troubleshooting 101: log to pflog

Rules with the **log** keyword log packet data to the [pflog](http://man.openbsd.org/pflog) device(s)

```shell
# log blocked packets
block log(all)

# logs initial packet of matching connections:	
pass log proto tcp to port ssh
   
# logs all matching packets:
pass log(all) proto tcp to port ssh log(all) 

# logs matches on this and all succeeding rules
pass log(matches) proto tcp to port ssh  

# logs all packets matches on this and all succeeding rules
pass log(all, matches) proto tcp to port ssh  
```


```shell
match log(all, matches) # log *everything*
```



---
## Troubleshooting 101: tcpdump, read from pflog

- [tcpdump](https://man.openbsd.org/tcpdump) is your friend

- Let it loose on the pflog device:

```shell
$ doas tcpdump -n -e -ttt -i pflog0 
tcpdump: WARNING: snaplen raised from 116 to 160
tcpdump: listening on pflog0, link-type PFLOG
May 29 21:06:27.165561 rule def/(match) pass in on bge1: 192.168.103.126.15526 >
213.187.179.198.22: . ack 2951513182 win 16332 (DF) [tos 0x10]
May 29 21:06:27.166934 rule 16/(match) pass in on bge0: 158.36.191.135.22 >
213.187.179.198.59516: . ack 1734404306 win 64800 [tos 0x8]
May 29 21:06:27.166939 rule 2/(match) match in on bge0: 158.36.191.135.22 >
213.187.179.198.59516: . ack 1 win 64800 [tos 0x8]
May 29 21:06:27.168340 rule def/(match) pass out on bge1: 213.187.179.198.22 >
192.168.103.126.15526: P 69:153(84) ack 0 win 17520 [tos 0x10]
May 29 21:06:27.169150 rule def/(match) pass out on bge1: 213.187.179.198.22 >
192.168.103.126.15526: P 153:333(180) ack 0 win 17520 [tos 0x10]
May 29 21:06:27.169265 rule def/(match) pass out on bge1: 213.187.179.198.22 >
```

- **NB** rule number, matches your *loaded* rule set

---
## Troubleshooting 101: Hitting and avoiding limits

- On busy systems, you may need to raise limits from default values

- Check with:

```shell
$ doas pfctl -s info
```
- versus the output of **pfctl -s memory** and **pfctl -s timeouts**

- You may need to bump up from defaults:

```shell
# increase state limit from 10'000 states on busy systems
set limit states 100000
# increase no of source nodes 
set limit src-nodes 100000
```


---
## Troubleshooting 101: netflow aka pflow (IPFIX)

- Records TCP/IP *flow* metadata
  - srcIP
  - dstIP 
  - (srcPort, dstPort)
  - startTime
  - endTime 
  - Packets
  - Bytes

- OpenBSD has the [pflow(4)](http://man.openbsd.org/pflow) virtual network interface
  - which generates the datagrams from the state table

- Useful for network monitoring, DDoS protection, etc.

---
## Troubleshooting 101: netflow setup

- Set up a *sensor*:

```shell
$ cat /etc/hostname.pflow0
flowsrc 192.168.103.1 flowdst 192.168.103.252:9995
pflowproto 10
```

- Then configure your *collector* at the **flowdst** IP address for analysis and network overlordship.

<br><br><br>
- Lots of collector options available in ports: nfsen, flow-tools, others.

- More info:
  - Michael W. Lucas: [Network Flow Analysis](https://www.nostarch.com/networkflow)
  - and Peter N. M. Hansteen: [Yes, You Too Can Be An Evil Network Overlord - On The Cheap With OpenBSD, pflow And nfsen](http://bsdly.blogspot.ca/2014/02/yes-you-too-can-be-evil-network.html).


---

